@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2015-2021 Mike Fährmann
+# Copyright 2015-2022 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -10,9 +10,11 @@
 
 from .common import GalleryExtractor, Extractor, Message
 from .nozomi import decode_nozomi
+from ..cache import memcache
 from .. import text, util
 import string
 import json
+import re
 
 
 class HitomiGalleryExtractor(GalleryExtractor):
@@ -24,32 +26,38 @@ class HitomiGalleryExtractor(GalleryExtractor):
                r"/(?:[^/?#]+-)?(\d+)")
     test = (
         ("https://hitomi.la/galleries/867789.html", {
-            "pattern": r"https://[a-c]b.hitomi.la/images/./../[0-9a-f]+.jpg",
-            "keyword": "4873ef9a523621fc857b114e0b2820ba4066e9ae",
+            "pattern": r"https://[a-c]a\.hitomi\.la/webp/\d+/\d+"
+                       r"/[0-9a-f]{64}\.webp",
+            "keyword": "4b584d09d535694d7d757c47daf5c15d116420d2",
+            "options": (("metadata", True),),
             "count": 16,
         }),
         # download test
         ("https://hitomi.la/galleries/1401410.html", {
             "range": "1",
-            "content": "b3ca8c6c8cc5826cf8b4ceb7252943abad7b8b4c",
+            "content": "d75d5a3d1302a48469016b20e53c26b714d17745",
         }),
         # Game CG with scenes (#321)
         ("https://hitomi.la/galleries/733697.html", {
-            "url": "e057652b40629d3d72b0ef059c6ec7556417139c",
             "count": 210,
         }),
         # fallback for galleries only available through /reader/ URLs
         ("https://hitomi.la/galleries/1045954.html", {
-            "url": "779b94b47d5f0c2341db03499270d2b5370196f6",
             "count": 1413,
         }),
         # gallery with "broken" redirect
         ("https://hitomi.la/cg/scathacha-sama-okuchi-ecchi-1291900.html", {
             "count": 10,
+            "options": (("format", "original"),),
+            "pattern": r"https://[a-c]b\.hitomi\.la/images/\d+/\d+"
+                       r"/[0-9a-f]{64}\.jpg",
         }),
         # no tags
         ("https://hitomi.la/cg/1615823.html", {
             "count": 22,
+            "options": (("format", "avif"),),
+            "pattern": r"https://[a-c]a\.hitomi\.la/avif/\d+/\d+"
+                       r"/[0-9a-f]{64}\.avif",
         }),
         ("https://hitomi.la/manga/amazon-no-hiyaku-867789.html"),
         ("https://hitomi.la/manga/867789.html"),
@@ -71,7 +79,7 @@ class HitomiGalleryExtractor(GalleryExtractor):
         self.info = info = json.loads(page.partition("=")[2])
 
         data = self._data_from_gallery_info(info)
-        if self.config("metadata", True):
+        if self.config("metadata", False):
             data.update(self._data_from_gallery_page(info))
         return data
 
@@ -133,19 +141,27 @@ class HitomiGalleryExtractor(GalleryExtractor):
         }
 
     def images(self, _):
+        # see https://ltn.hitomi.la/gg.js
+        gg_m, gg_b, gg_default = _parse_gg(self)
+
+        fmt = self.config("format") or "webp"
+        if fmt == "original":
+            subdomain, fmt, ext = "b", "images", None
+        else:
+            subdomain, ext = "a", fmt
+
         result = []
         for image in self.info["files"]:
             ihash = image["hash"]
             idata = text.nameext_from_url(image["name"])
+            if ext:
+                idata["extension"] = ext
 
             # see https://ltn.hitomi.la/common.js
-            inum = int(ihash[-3:-1], 16)
-            offset = 2 if inum < 0x44 else 1 if inum < 0x88 else 0
-
-            url = "https://{}b.hitomi.la/images/{}/{}/{}.{}".format(
-                chr(97 + offset),
-                ihash[-1], ihash[-3:-1], ihash,
-                idata["extension"],
+            inum = int(ihash[-1] + ihash[-3:-1], 16)
+            url = "https://{}{}.hitomi.la/{}/{}/{}/{}.{}".format(
+                chr(97 + gg_m.get(inum, gg_default)),
+                subdomain, fmt, gg_b, inum, ihash, idata["extension"],
             )
             result.append((url, idata))
         return result
@@ -155,6 +171,7 @@ class HitomiTagExtractor(Extractor):
     """Extractor for galleries from tag searches on hitomi.la"""
     category = "hitomi"
     subcategory = "tag"
+    root = "https://hitomi.la"
     pattern = (r"(?:https?://)?hitomi\.la/"
                r"(tag|artist|group|series|type|character)/"
                r"([^/?#]+)\.html")
@@ -179,9 +196,54 @@ class HitomiTagExtractor(Extractor):
             self.tag = tag
 
     def items(self):
-        url = "https://ltn.hitomi.la/{}/{}.nozomi".format(self.type, self.tag)
         data = {"_extractor": HitomiGalleryExtractor}
+        nozomi_url = "https://ltn.hitomi.la/{}/{}.nozomi".format(
+            self.type, self.tag)
+        headers = {
+            "Origin": self.root,
+            "Cache-Control": "max-age=0",
+        }
 
-        for gallery_id in decode_nozomi(self.request(url).content):
-            url = "https://hitomi.la/galleries/{}.html".format(gallery_id)
-            yield Message.Queue, url, data
+        offset = 0
+        while True:
+            headers["Referer"] = "{}/{}/{}.html?page={}".format(
+                self.root, self.type, self.tag, offset // 100 + 1)
+            headers["Range"] = "bytes={}-{}".format(offset, offset+99)
+            nozomi = self.request(nozomi_url, headers=headers).content
+
+            for gallery_id in decode_nozomi(nozomi):
+                gallery_url = "{}/galleries/{}.html".format(
+                    self.root, gallery_id)
+                yield Message.Queue, gallery_url, data
+
+            if len(nozomi) < 100:
+                return
+            offset += 100
+
+
+@memcache()
+def _parse_gg(extr):
+    page = extr.request("https://ltn.hitomi.la/gg.js").text
+
+    m = {}
+
+    keys = []
+    for match in re.finditer(
+            r"case\s+(\d+):(?:\s*o\s*=\s*(\d+))?", page):
+        key, value = match.groups()
+        keys.append(int(key))
+
+        if value:
+            value = int(value)
+            for key in keys:
+                m[key] = value
+            keys.clear()
+
+    for match in re.finditer(
+            r"if\s+\(g\s*===?\s*(\d+)\)[\s{]*o\s*=\s*(\d+)", page):
+        m[int(match.group(1))] = int(match.group(2))
+
+    d = re.search(r"(?:var\s|default:)\s*o\s*=\s*(\d+)", page)
+    b = re.search(r"b:\s*[\"'](.+)[\"']", page)
+
+    return m, b.group(1).strip("/"), int(d.group(1)) if d else 1
