@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2017-2021 Mike Fährmann
+# Copyright 2017-2022 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -12,6 +12,7 @@ import re
 import os
 import sys
 import json
+import time
 import random
 import sqlite3
 import binascii
@@ -20,6 +21,7 @@ import functools
 import itertools
 import urllib.parse
 from http.cookiejar import Cookie
+from email.utils import mktime_tz, parsedate_tz
 from . import text, exception
 
 
@@ -81,6 +83,16 @@ def identity(x):
     return x
 
 
+def true(_):
+    """Always returns True"""
+    return True
+
+
+def false(_):
+    """Always returns False"""
+    return False
+
+
 def noop():
     """Does nothing"""
 
@@ -262,6 +274,15 @@ def remove_directory(path):
         pass
 
 
+def set_mtime(path, mtime):
+    try:
+        if isinstance(mtime, str):
+            mtime = mktime_tz(parsedate_tz(mtime))
+        os.utime(path, (time.time(), mtime))
+    except Exception:
+        pass
+
+
 def load_cookiestxt(fp):
     """Parse a Netscape cookies.txt file and return a list of its Cookies"""
     cookies = []
@@ -403,6 +424,7 @@ GLOBALS = {
     "parse_int": text.parse_int,
     "urlsplit" : urllib.parse.urlsplit,
     "datetime" : datetime.datetime,
+    "timedelta": datetime.timedelta,
     "abort"    : raises(exception.StopExtraction),
     "terminate": raises(exception.TerminateExtraction),
     "re"       : re,
@@ -418,18 +440,106 @@ def build_duration_func(duration, min=0.0):
     if not duration:
         return None
 
-    try:
-        lower, upper = duration
-    except TypeError:
-        pass
+    if isinstance(duration, str):
+        lower, _, upper = duration.partition("-")
+        lower = float(lower)
     else:
+        try:
+            lower, upper = duration
+        except TypeError:
+            lower, upper = duration, None
+
+    if upper:
+        upper = float(upper)
         return functools.partial(
             random.uniform,
             lower if lower > min else min,
             upper if upper > min else min,
         )
+    else:
+        if lower < min:
+            lower = min
+        return lambda: lower
 
-    return functools.partial(identity, duration if duration > min else min)
+
+def build_extractor_filter(categories, negate=True, special=None):
+    """Build a function that takes an Extractor class as argument
+    and returns True if that class is allowed by 'categories'
+    """
+    if isinstance(categories, str):
+        categories = categories.split(",")
+
+    catset = set()  # set of categories / basecategories
+    subset = set()  # set of subcategories
+    catsub = []     # list of category-subcategory pairs
+
+    for item in categories:
+        category, _, subcategory = item.partition(":")
+        if category and category != "*":
+            if subcategory and subcategory != "*":
+                catsub.append((category, subcategory))
+            else:
+                catset.add(category)
+        elif subcategory and subcategory != "*":
+            subset.add(subcategory)
+
+    if special:
+        catset |= special
+    elif not catset and not subset and not catsub:
+        return true if negate else false
+
+    tests = []
+
+    if negate:
+        if catset:
+            tests.append(lambda extr:
+                         extr.category not in catset and
+                         extr.basecategory not in catset)
+        if subset:
+            tests.append(lambda extr: extr.subcategory not in subset)
+    else:
+        if catset:
+            tests.append(lambda extr:
+                         extr.category in catset or
+                         extr.basecategory in catset)
+        if subset:
+            tests.append(lambda extr: extr.subcategory in subset)
+
+    if catsub:
+        def test(extr):
+            for category, subcategory in catsub:
+                if category in (extr.category, extr.basecategory) and \
+                        subcategory == extr.subcategory:
+                    return not negate
+            return negate
+        tests.append(test)
+
+    if len(tests) == 1:
+        return tests[0]
+    if negate:
+        return lambda extr: all(t(extr) for t in tests)
+    else:
+        return lambda extr: any(t(extr) for t in tests)
+
+
+def build_proxy_map(proxies, log=None):
+    """Generate a proxy map"""
+    if not proxies:
+        return None
+
+    if isinstance(proxies, str):
+        if "://" not in proxies:
+            proxies = "http://" + proxies.lstrip("/")
+        return {"http": proxies, "https": proxies}
+
+    if isinstance(proxies, dict):
+        for scheme, proxy in proxies.items():
+            if "://" not in proxy:
+                proxies[scheme] = "http://" + proxy.lstrip("/")
+        return proxies
+
+    if log:
+        log.warning("invalid proxy specifier: %s", proxies)
 
 
 def build_predicate(predicates):
